-----------------------------------------------------------------------
--
--  File:        ms-dos.ads
--  Description: Interface to DJGPP v2.0 MS-DOS library
--  Rev:         0.1
--  Date:        03/09/96
--  Author:      Jerry van Dijk
--  Mail:        jerry@jvdsys.nextjk.stuyts.nl
--
--  Copyright (c) Jerry van Dijk, 1996
--  Forelstraat 211
--  2037 KV  HAARLEM
--  THE NETHERLANDS
--  tel int + 31 23 540 1052
--
--  Permission granted to use for any purpose, provided this copyright
--  remains attached and unmodified.
--
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--
-----------------------------------------------------------------------

with Unchecked_Conversion;

with System; use System;
with Interfaces; use Interfaces;

package MS_DOS is

   -------------------------
   -- Processor registers --
   -------------------------

   type Dpmi_Regs is
      record
         Di     : Unsigned_16;
         Di_Hi  : Unsigned_16;
         Si     : Unsigned_16;
         Si_Hi  : Unsigned_16;
         Bp     : Unsigned_16;
         Bp_Hi  : Unsigned_16;
         Res    : Unsigned_16;
         Res_Hi : Unsigned_16;
         Bx     : Unsigned_16;
         Bx_Hi  : Unsigned_16;
         Dx     : Unsigned_16;
         Dx_Hi  : Unsigned_16;
         Cx     : Unsigned_16;
         Cx_Hi  : Unsigned_16;
         Ax     : Unsigned_16;
         Ax_Hi  : Unsigned_16;
         Flags  : Unsigned_16;
         Es     : Unsigned_16;
         Ds     : Unsigned_16;
         Fs     : Unsigned_16;
         Gs     : Unsigned_16;
         Ip     : Unsigned_16;
         Cs     : Unsigned_16;
         Sp     : Unsigned_16;
         Ss     : Unsigned_16;
      end record;
   pragma Convention (C, Dpmi_Regs);


   ------------------------------
   -- DOS extender information --
   ------------------------------

   type Go32_Info_Block is
      record
         Size_Of_This_Structure_In_Bytes       : Unsigned_32;
         Linear_Address_Of_Primary_Screen      : Unsigned_32;
         Linear_Address_Of_Secondary_Screen    : Unsigned_32;
         Linear_Address_Of_Transfer_Buffer     : Unsigned_32;
         Size_Of_Transfer_Buffer               : Unsigned_32;
         Pid                                   : Unsigned_32;
         Master_Interrupt_Controller_Base      : Unsigned_8;
         Slave_Interrupt_Controller_Base       : Unsigned_8;
         Selector_For_Linear_Memory            : Unsigned_16;
         Linear_Address_Of_Stub_Info_Structure : Unsigned_32;
         Linear_Address_Of_Original_Psp        : Unsigned_32;
         Run_Mode                              : Unsigned_16;
         Run_Mode_Info                         : Unsigned_16;
      end record;
   pragma Convention (C, Go32_Info_Block);

   DOS_Extender : Go32_Info_Block;
   pragma Import (C, DOS_Extender, "_go32_info_block");


   --------------------------------------------------------
   -- Handy synomiems for long winded DOS extender names --
   --------------------------------------------------------

   TB_Size : Unsigned_32 renames DOS_Extender.Size_Of_Transfer_Buffer;
   --  Current size of the real-mode transfer buffer

   TB_Address : Unsigned_32 renames
     DOS_Extender.Linear_Address_Of_Transfer_Buffer;
   --  Address of the real-mode transfer buffer

   DOS_Ds : Unsigned_16 renames DOS_Extender.Selector_For_Linear_Memory;
   --  Selector for first Mb of memory


   --------------------------------
   -- Call a real-mode interrupt --
   --------------------------------

   procedure Dpmi_Int (Vector : Unsigned_16; Regs : in out Dpmi_Regs);
   pragma Import (C, Dpmi_Int, "__dpmi_int");


   ----------------------------------------
   -- Move data from/to real-mode memory --
   ----------------------------------------

   procedure Dosmemget (Offset : Unsigned_32;
                        Length : Unsigned_32;
                        Buffer : Unsigned_32);
   pragma Import (C, Dosmemget, "dosmemget");


   -----------------------------
   -- Access real-mode memory --
   -----------------------------

   procedure Set_Selector (Selector : Unsigned_16);
   pragma Import (C, Set_Selector, "_farsetsel");

   function Peek_Byte (Offset : Unsigned_32) return Unsigned_8;
   pragma Import (C, Peek_Byte, "_farnspeekb");

   function Peek_Word (Offset : Unsigned_32) return Unsigned_16;
   pragma Import (C, Peek_Word, "_farnspeekw");

   procedure Poke_Byte (Offset : Unsigned_32; Value : Unsigned_8);
   pragma Import (C, Poke_Byte, "_farnspokeb");


   ---------------------------
   -- Access hardware ports --
   ---------------------------

   function Inport_Byte (Port : Unsigned_16) return Unsigned_8;
   pragma Import (C, Inport_Byte, "inportb");

   procedure Outport_Byte (Port : Unsigned_16; Value : Unsigned_8);
   pragma Import (C, Outport_Byte, "outportb");


   -----------------------------------------------------------
   -- Convert a linear adress into its real-mode equivalent --
   -----------------------------------------------------------

   function RM_Offset (Address : Unsigned_32) return Unsigned_16;
   function RM_Segment (Address : Unsigned_32) return Unsigned_16;


   ---------------------------------------------------------
   -- Convert a DWORD real-mode address into a linear one --
   ---------------------------------------------------------

   function RM_To_Linear (Address : Unsigned_32) return Unsigned_32;


   ----------------------------------------------------
   -- Convert a system address into a linear address --
   ----------------------------------------------------

   function To_Unsigned_32 is new Unchecked_Conversion (Address, Unsigned_32);

end MS_DOS;