-----------------------------------------------------------------------
--
--  File:        vesa_12.ads
--  Description: Interface to VESA v1.2 driver
--  Rev:         0.1
--  Date:        03/09/96
--  Author:      Jerry van Dijk
--  Mail:        jerry@jvdsys.nextjk.stuyts.nl
--
--  Copyright (c) Jerry van Dijk, 1996
--  Forelstraat 211
--  2037 KV  HAARLEM
--  THE NETHERLANDS
--  tel int + 31 23 540 1052
--
--  Permission granted to use for any purpose, provided this copyright
--  remains attached and unmodified.
--
--  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
--  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
--  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
--
-----------------------------------------------------------------------

with Interfaces; use Interfaces;

package VESA_12 is

   -----------------------------
   -- VESA Driver information --
   -----------------------------

   type VgaInfoBlock is
      record
         VESASignature : Unsigned_32; -- 4 signature bytes
         VESAVersion   : Unsigned_16; -- VESA version number
         OEMStringPtr  : Unsigned_32; -- Pointer to OEM string
         Capabilities  : Unsigned_32; -- capabilities of the video environment
         VideoModePtr  : Unsigned_32; -- pointer to supported Super VGA modes
         TotalMemory   : Unsigned_16; -- Number of 64kb memory blocks on board
      end record;
   pragma Pack (VgaInfoBlock);


   ------------------------------------
   -- VESA graphics mode information --
   ------------------------------------

   type ModeInfoBlock is
      record
        ModeAttributes      : Unsigned_16; -- mode attributes
        WinAAttributes      : Unsigned_8;  -- window A attributes
        WinBAttributes      : Unsigned_8;  -- window B attributes
        WinGranularity      : Unsigned_16; -- window granularity
        WinSize             : Unsigned_16; -- window size
        WinASegment         : Unsigned_16; -- window A start segment
        WinBSegment         : Unsigned_16; -- window B start segment
        WinFuncPtr          : Unsigned_32; -- pointer to windor function
        BytesPerScanLine    : Unsigned_16; -- bytes per scan line
        XResolution         : Unsigned_16; -- horizontal resolution
        YResolution         : Unsigned_16; -- vertical resolution
        XCharSize           : Unsigned_8;  -- character cell width
        YCharSize           : Unsigned_8;  -- character cell height
        NumberOfPlanes      : Unsigned_8;  -- number of memory planes
        BitsPerPixel        : Unsigned_8;  -- bits per pixel
        NumberOfBanks       : Unsigned_8;  -- number of banks
        MemoryModel         : Unsigned_8;  -- memory model type
        BankSize            : Unsigned_8;  -- bank size in kb
        NumberOfImagePages  : Unsigned_8;  -- number of images
        Reserved            : Unsigned_8;  -- reserved for page function
        RedMaskSize         : Unsigned_8;  -- size direct color red mask
        RedFieldPosition    : Unsigned_8;  -- bit position of LSB of red mask
        GreenMaskSize       : Unsigned_8;  -- size direct color green mask
        GreenFieldPosition  : Unsigned_8;  -- bit position of LSB of green mask
        BlueMaskSize        : Unsigned_8;  -- size direct color blue mask
        BlueFieldPosition   : Unsigned_8;  -- bit position of LSB of blue mask
        RsvdMaskSize        : Unsigned_8;  -- size direct color reserved mask
        DirectColorModeInfo : Unsigned_8;  -- Direct Color mode attributes
      end record;
   pragma Pack (ModeInfoBlock);


   ------------------------------
   -- VESA v1.2 function calls --
   ------------------------------

   procedure SetSVGAMode (Mode : Unsigned_16);
   function GetCurrentVideoMode return Unsigned_16;
   procedure GetVgaInfoBlock (Info : in out VgaInfoBlock);
   procedure SetVideoMemoryWindow (Window : Unsigned_16; Pos : Unsigned_16);
   procedure GetModeInfoBlock (Mode : Unsigned_16; Info : out ModeInfoBlock);


   ------------------------
   -- Package exceptions --
   ------------------------

   VESA_Error : exception;    -- a VESA call failed

end VESA_12;